# htmltemplate.rb
#
# An emulation layer on top of PT to mimic HTML::Template
#

############################################################

class PageTemplate
  class HTGlossary < SyntaxGlossary
    @directive = /<!--(.+?)-->/m

    default { |command,parser|
      TextCommand.new("<!--#{command}-->",parser)
    }

    define(/^tmpl_var (?:name=)?"?((?:\w+)(?:\.\w+\??)*)"?(?:\s+processor="?(\w+)"?)?$/i) {
      |match,parser|
      # Value, Preprocessor, Parser
      ValueCommand.new(match[1],match[2],parser)
    }

    define(/^(tmpl_if|tmpl_unless) (?:name=)?"?((\w+)(\.\w+\??)*)"?$/i) {
      |match,parser|
      # Called_As, Value, parser
      IfCommand.new(match[1],match[2],parser)
    }

    define(/^(tmpl_loop) (?:name=)"?(\w+(?:\.\w+\??)*)"?(?:\s+iterators?="?((?:\s*\w+)+)"?)?$/i) {
      |match,parser|
      # Called_As, Value, Iterators
      LoopCommand.new(match[1],match[2],match[3])
    }

    define(/^tmpl_include (?:name=|file=)?"?((\w+)(?:\.\w+\??)*)"?$/i) {
      |match,parser|
      # Value, parser
      IncludeCommand.new(match[1],parser)
    }

    # Command#else's expect only to be called
    modifier(:else) { |cmd,command|
      if command =~ /^tmpl_else$/i
        cmd.else
        true
      else
        false
      end
    }
    modifier(:end) { |cmd,command|
      if command =~ /^\/\s*#{cmd.called_as}$/i
        cmd.end
        true
      else
        false
      end
    }
    modifier(:when) { |cmd,command|
      case command
      when /^tmpl_when\s+(?:name=)"?(\w(?:.\w+\??)*)"?$/i
        cmd.when($1)
        true
      when /^tmpl_else$/i
        cmd.else($1)
        true
      else
        false
      end
    }
  end
end
